@@ -5,7 +5,7 @@
 #' columns.
 #'
 #' Unlike other dplyr verbs, `arrange()` largely ignores grouping; you
-#' need to explicitly mention grouping variables (or use  `by_group = TRUE`)
+#' need to explicitly mention grouping variables (or use  `.by_group = TRUE`)
 #' in order to group by them, and functions of variables are evaluated
 #' once per data frame, not once per group.
 #'
@@ -116,7 +116,24 @@ arrange_rows <- function(.data, dots) {
   data <- withCallingHandlers({
     transmute(new_data_frame(.data), !!!quosures)
   }, error = function(cnd) {
-    stop_arrange_transmute(cnd)
+    if (inherits(cnd, "dplyr:::mutate_error")) {
+      error_name <- cnd$error_name
+      index <- sub("^.*_", "", error_name)
+      error_expression <- cnd$error_expression
+
+      bullets <- c(
+        x = glue("Could not create a temporary column for `..{index}`."),
+        i = glue("`..{index}` is `{error_expression}`.")
+      )
+    } else {
+      bullets <- c(x = conditionMessage(cnd))
+    }
+
+    abort(c(
+      "arrange() failed at implicit mutate() step. ",
+      bullets
+    ), class = "dplyr_error")
+
   })
 
   # we can't just use vec_compare_proxy(data) because we need to apply
@@ -140,19 +157,3 @@ arrange_rows <- function(.data, dots) {
 
   exec("order", !!!unname(proxies), decreasing = FALSE, na.last = TRUE)
 }
-
-# FIXME: Temporary util until the API change from
-# https://github.com/r-lib/vctrs/pull/1155 is on CRAN and we can
-# depend on it
-delayedAssign(
-  "dplyr_proxy_order",
-  if (env_has(ns_env("vctrs"), "vec_proxy_order")) {
-    vec_proxy_order
-  } else {
-    function(x, ...) vec_proxy_compare(x, ..., relax = TRUE)
-  }
-)
-
-# Hack to pass CRAN check with older vctrs versions where
-# `vec_proxy_order()` doesn't exist
-utils::globalVariables("vec_proxy_order")
