@@ -264,7 +264,7 @@ mutate_cols <- function(.data, ...) {
             # same error as would have been generated by mask$eval_all_mutate()
             group <- wrong[1L]
             mask$set_current_group(group)
-            stop_mutate_recycle_incompatible_size(sizes[group])
+            abort(x_size = sizes[group], class = "dplyr:::mutate_incompatible_size")
           }
         }
       }
@@ -279,9 +279,7 @@ mutate_cols <- function(.data, ...) {
       if (is.null(chunks)) {
         if (!is.null(dots_names) && dots_names[i] != "") {
           new_columns[[dots_names[i]]] <- zap()
-
-          # we might get a warning if dots_names[i] does not exist
-          suppressWarnings(mask$remove(dots_names[i]))
+          mask$remove(dots_names[i])
         }
         next
       }
@@ -322,23 +320,65 @@ mutate_cols <- function(.data, ...) {
 
   },
   error = function(e) {
-    if (inherits(e, "rlang_error_data_pronoun_not_found")) {
-      stop_error_data_pronoun_not_found(conditionMessage(e), index = i, dots = dots, fn = "mutate")
-    } else if (inherits(e, "dplyr:::mutate_incompatible_size")) {
-      e$size <- vec_size(rows[[i]])
-      stop_mutate_recycle_incompatible_size(e, index = i, dots = dots)
+    local_call_step(dots = dots, .index = i, .fn = "mutate", .dot_data = inherits(e, "rlang_error_data_pronoun_not_found"))
+    call_step_envir <- peek_call_step()
+    error_name <- call_step_envir$error_name
+    error_expression <- call_step_envir$error_expression
+
+    show_group_details <- TRUE
+    if (inherits(e, "dplyr:::mutate_incompatible_size")) {
+      size <- vec_size(rows[[i]])
+      x_size <- e$x_size
+      bullets <- c(
+        x = glue("Input `{error_name}` can't be recycled to size {size}."),
+        i = cnd_bullet_input_info(),
+        i = glue("Input `{error_name}` must be size {or_1(size)}, not {x_size}."),
+        i = cnd_bullet_rowwise_unlist()
+      )
     } else if (inherits(e, "dplyr:::mutate_mixed_null")) {
-      stop_mutate_mixed_null(index = i, dots = dots)
+      show_group_details <- FALSE
+      bullets <- c(
+        x = glue("`{error_name}` must return compatible vectors across groups."),
+        i = cnd_bullet_input_info(),
+        i = "Cannot combine NULL and non NULL results.",
+        i = cnd_bullet_rowwise_unlist()
+      )
     } else if (inherits(e, "dplyr:::mutate_not_vector")) {
-      stop_mutate_not_vector(index = i, dots = dots, result = e$result)
+      bullets <- c(
+        x = glue("Input `{error_name}` must be a vector, not {friendly_type_of(e$result)}."),
+        i = cnd_bullet_input_info(),
+        i = cnd_bullet_rowwise_unlist()
+      )
     } else if(inherits(e, "dplyr:::error_mutate_incompatible_combine")) {
-      stop_combine(e$parent, index = i, dots = dots, fn = "mutate")
+      show_group_details <- FALSE
+      bullets <- c(
+        x = glue("Input `{error_name}` must return compatible vectors across groups"),
+        i = cnd_bullet_input_info(),
+        i = cnd_bullet_combine_details(e$parent$x, e$parent$x_arg),
+        i = cnd_bullet_combine_details(e$parent$y, e$parent$y_arg)
+      )
     } else {
-      stop_dplyr(i, dots, fn = "mutate", problem = conditionMessage(e), parent = e)
+      bullets <- c(
+        x = conditionMessage(e), i = cnd_bullet_input_info()
+      )
     }
+
+    abort(c(
+      cnd_bullet_header(),
+      bullets,
+      i = if(show_group_details) cnd_bullet_cur_group_label()
+    ), class = c("dplyr:::mutate_error", "dplyr_error"), error_name = error_name, error_expression = error_expression)
+
   },
   warning = function(w) {
-    warn_dplyr(i, dots, fn = "mutate", problem = conditionMessage(w), parent = w)
+    local_call_step(dots = dots, .index = i, .fn = "mutate")
+
+    warn(c(
+      cnd_bullet_header(),
+      i = conditionMessage(w),
+      i = cnd_bullet_input_info(),
+      i = cnd_bullet_cur_group_label()
+    ))
   })
 
   is_zap <- map_lgl(new_columns, inherits, "rlang_zap")
